//*************************************************************************
//* --------------------
//* DOMPMTPhotoCathode
//* --------------------
//* (Description)
//*	Abstract base class for DOM modules.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2003/12/22  K.Hoshina	Original version.
//*************************************************************************

#include <TRandom.h>
#include "DOMPMT.h"
 
#include "DOMPMTPhotoCathode.h"
#include "DOMOpticalPhoton.h"
#include "DOMPhotoElectron.h"

//#define __DOMABSOLUTEQE__  0.3
#define __DOMABSOLUTEQE__  1 
#define __DOMZOFFSET__     8.4     //mm 
#define __DOMPMTR__        136.69  //mm 

const Int_t    DOMPMTPhotoCathode::fgkNXbins = 30;
const Int_t    DOMPMTPhotoCathode::fgkNYbins = 30;
const Double_t DOMPMTPhotoCathode::fgkXmin   = -15.;
const Double_t DOMPMTPhotoCathode::fgkXmax   = 15.;
const Double_t DOMPMTPhotoCathode::fgkYmin   = -15.;
const Double_t DOMPMTPhotoCathode::fgkYmax   = 15.;

ClassImp(DOMPMTPhotoCathode)

using namespace std;

//=====================================================================
//* constructor -------------------------------------------------------

DOMPMTPhotoCathode::DOMPMTPhotoCathode(const TString &uniformity,
                                       const Char_t  *name, 
                                       const Char_t *title)
                   : DOMVModule(name,title), fUniformityMap(0), fPEMap(0),
                     fPhotonArray(0), fPEArray(1), fAbsoluteQE(0) ,fCanvas(0)
{
   fUniformityData = uniformity;
   SetIOStyle(kDirectConnect);
}

//=====================================================================
//* destructor --------------------------------------------------------

DOMPMTPhotoCathode::~DOMPMTPhotoCathode()
{
   if (fUniformityMap) delete fUniformityMap;
   if (fPEMap)         delete fPEMap;
   if (fPhotonArray)   delete fPhotonArray;
}

//===================================================================
//* Configure ---------------------------------------------------------

void DOMPMTPhotoCathode::Configure()
{

   CreateUniformityMap();

   fPEMap = new TH2D("PhotoElectronMap", "PhotoElectronMap",
                              fgkNXbins, fgkXmin, fgkXmax, 
                              fgkNYbins, fgkYmin, fgkYmax );

   fAbsoluteQE = __DOMABSOLUTEQE__;

}

//=====================================================================
//* Process -----------------------------------------------------------

void DOMPMTPhotoCathode::Process()
{

   // delete remaining arrays
   if (fPhotonArray) fPhotonArray->Clear();
   fPEArray.Clear();

   // input photon map

   if (GetIOStyle() == kASCII) {
#if 0
      if (!(fIfs.open(GetInputFileName().Data()))) {
         cerr << "DOMPMTPhotoCathode::Configure: file " << GetInputFileName() 
              << " open failed! abort." << endl;
         abort();
      }
#endif

      TVector3 pos, dir;
      Double_t edep, tof;
      while ( *fIfs >> pos(0) >> pos(1) >> pos(2) >> edep >> tof) {
         DOMOpticalPhoton *photon = new DOMOpticalPhoton(pos, dir);
         fPhotonArray->Add(photon);
      }
      fPhotonArray->SetOwner();

   } else if (GetIOStyle() == kDirectConnect) {
      fPhotonArray = new TObjArray(((DOMPMT *)GetMother())->GetPhotonArray());
   }


   // get p.e
  
   //fOfs.open(const_cast<Char_t*>(GetOutputFileName().Data()));
   TIter next(fPhotonArray);
   DOMOpticalPhoton *ph; 
   Int_t i = 0;
   while ((ph = (DOMOpticalPhoton *)next())) {

      if (ph->IsValid()) {
         // the photon is alive!   
         TVector3 pos        = ph->GetPosition();
         Double_t x          = GetXPosition(pos) * 0.1; // mm -> cm
         Double_t y          = GetYPosition(pos) * 0.1; 
         Int_t    bin        = fUniformityMap->FindBin(x, y); 
         Double_t uniformity = fUniformityMap->GetBinContent(bin);

         Double_t probable   = fAbsoluteQE * uniformity;
         Double_t rand       = 100 * gRandom->Uniform(0., 1.); // percent
         cerr << "x, y, p, rand = " 
              << x << " " << y << "     " << probable << " " << rand << endl;

         if (rand < probable) {

            cerr << "DOMPMTPhotoCathode::Process: Congraturation!!! "
                 << "1 p.e generated! " << endl;
            DOMPhotoElectron *pe = new DOMPhotoElectron(ph->GetPosition(), 
                                                        ph->GetDirection());
            fPEArray.Add(pe); 
            //pe->Output(fOfs); 
            fPEMap->Fill(x, y);  

         } else {
            cerr << "DOMPMTPhotoCathode::Process: Sorry! "
                 << "All photons are absorbed. " << endl;
            ChangeState(kAbsorbed);
         }
      }      
      i++;
   }

   fPEArray.SetOwner();
   ((DOMPMT *)GetMother())->SetPhotoElectronArray(&fPEArray);
   
}

//=====================================================================
//* Finish ------------------------------------------------------------

void DOMPMTPhotoCathode::Finish()
{
   fCanvas = new TCanvas("Uniformity_PEMap", "Uniformity and PE", 
                          10, 10, 800, 400);
   fCanvas->Divide(2, 1); 
   fCanvas->cd(1);
   fUniformityMap->Draw("LEGO2");
   fCanvas->cd(2);
   fPEMap->Draw("LEGO2");
}

//=====================================================================
//* CreateUniformityMap -----------------------------------------------

void DOMPMTPhotoCathode::CreateUniformityMap()
{
   // uniformity data open

   ifstream in(const_cast<Char_t*>(fUniformityData.Data()));

   if (!in.good()) {
      cerr << "DOMPMTPhotoCathode::CreateUniformityMap: file " << fUniformityData
           << " open failed! abort." << endl;
      abort();
   } 

   fUniformityMap = new TH2D("UniformityMap", "UniformityMap",
                              fgkNXbins, fgkXmin, fgkXmax, 
                              fgkNYbins, fgkYmin, fgkYmax );

   Double_t   x, y, weight;

   while (in >> x >> y >> weight) {  
      if (weight) {

         Int_t    bin = fUniformityMap->FindBin(x, y);
         Double_t content = fUniformityMap->GetBinContent(bin);

         if (content) {
            weight = 0.5 * (content + weight); 
            fUniformityMap->SetBinContent(bin, weight);
         } else {
            fUniformityMap->Fill(x, y, weight);  
         }
#if 0
         cerr << "bin, x, y, weight, content = " 
              << bin << " " << x << " " << y << " " 
              << weight << " " << content << endl;
#endif
      }
   }

   in.close();
   in.clear();


}

//=====================================================================
//* GetXPosition -------------------------------------------------

Double_t DOMPMTPhotoCathode::GetXPosition(TVector3 gp)
{

   TVector3 orig(0, 0, __DOMZOFFSET__);
   TVector3 xh(gp.X(), 0, gp.Z());

   Double_t xtheta = (xh - orig).Theta();  

   Double_t x = __DOMPMTR__ * xtheta;

   if (gp.X() < 0) x = -x;

   return x;

}

//=====================================================================
//* GetYPosition -------------------------------------------------

Double_t DOMPMTPhotoCathode::GetYPosition(TVector3 gp)
{

   TVector3 orig(0, 0, __DOMZOFFSET__);
   TVector3 yh(0, gp.Y(), gp.Z());

   Double_t ytheta = (yh - orig).Theta();  

   Double_t y = __DOMPMTR__ * ytheta;

   if (gp.Y() < 0) y = -y;

   return y;

}
