//*************************************************************************
//* --------------------
//* DOMPMTWaveForm
//* --------------------
//* (Description)
//*	Abstract base class for DOM modules.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2003/12/22  K.Hoshina	Original version.
//*************************************************************************

#include "DOMPMTWaveForm.h"
#include "DOMPMTPEChargeDatum.h"
#include "DOMPMT.h"

#include <iostream>

ClassImp(DOMPMTWaveForm)

using namespace std;

// TMP_TREATMENT!!!
// gain curve possibly deviate among each PMTs!

const Double_t DOMPMTWaveForm::fgkSigma      = 2.18659; // ns
const Double_t DOMPMTWaveForm::fgkTerminator = 50;      // 50 ohm
const Double_t DOMPMTWaveForm::fgkT0         = 20;      // 10 ns 
const Double_t DOMPMTWaveForm::fgkGround     = 0;       // ground level
const Double_t DOMPMTWaveForm::fgkBinMax     = 100;     // ground level

// global function

//================================================================
// function GaussianWaveForm
//================================================================

Double_t GaussianWaveForm(Double_t *x, Double_t *par) { 

  Double_t norm   = par[0];
  Double_t sigma  = par[1];
  Double_t t0     = par[2];
  Double_t ground = par[3];
  Double_t signal = - norm / (TMath::Sqrt(2 * TMath::Pi()) * sigma)
                    * exp(- (x[0] - t0) * (x[0] - t0) / (2 * sigma * sigma)) 
                    + ground;

   return signal; 

}




//=====================================================================
//* constructor -------------------------------------------------------

DOMPMTWaveForm::DOMPMTWaveForm( const Char_t  *name, 
                                const Char_t  *title)
                   : DOMVModule(name,title), fSinglePEChargeArray(0),
                     fWaveForm(0), fNormHist(0), fCanvas(0), fCanvas1(0)
{
   SetIOStyle(kDirectConnect);
}

//=====================================================================
//* destructor --------------------------------------------------------

DOMPMTWaveForm::~DOMPMTWaveForm()
{
   if (fSinglePEChargeArray) delete fSinglePEChargeArray;
   if (fWaveForm)            delete fWaveForm;
   if (fCanvas)              delete fCanvas;
}

//=====================================================================
//* GetNormalization --------------------------------------------------

Double_t DOMPMTWaveForm::GetNormalization(Double_t charge) 
{
   
   // V[V] = I[C/s] * R[Ohm];
   // integV = integ(I[C/s] * R[Ohm] ds) = Normarization;
   
   // V.s = C/s*s * ohm ;
   //     = C/ns/1e-09 * ohm * ;
   //     = 
   // K = (C * R) / V.s 
   // 
   Double_t norm = (charge * fgkTerminator) / (1e-03 * 1e-09) ; 

   cerr << "DOMPMTWaveForm:: charge, current normalization = " 
        << charge << " " << norm  << endl;
   return norm; 
}

//===================================================================
//* Configure ---------------------------------------------------------

void DOMPMTWaveForm::Configure()
{
   // Read calibration data

   fCanvas = new TCanvas("WaveForm", "WaveForm of single pe", 10, 10, 600, 400);
   fCanvas->cd();

   fNormHist = new TH1D("Normalization" , "Normalization of simulated pulse", 
                         500, 0, 5000);

}

//=====================================================================
//* Process -----------------------------------------------------------

void DOMPMTWaveForm::Process()
{
   cerr << "DOMPMTWaveForm::Process is called! " << endl;

   // delete remaining arrays
   if (fSinglePEChargeArray) fSinglePEChargeArray->Clear();

   //
   // read photo electrons 
   //

   if (GetIOStyle() == kASCII) {
      // input file open
#if 0
      if (!fIfs.open(GetInputFileName())) {
         cerr << "DOMPMTWaveForm::Configure: file " << GetInputFileName() 
              << " open failed! abort." << endl;
         abort();
      } 
      cerr << "DOMPMTWaveForm::Process: "
           << "Sorry, file I/O option is not yet implimented!" << endl; 
      abort();
#endif

   } else if (GetIOStyle() == kDirectConnect) {

      fSinglePEChargeArray = 
       new TObjArray(((DOMPMT *)GetMother())->GetSinglePEChargeArray());

   }  

   //
   // create charge responce
   //

   if (fSinglePEChargeArray->GetEntries() == 1) {
      // single p.e event!
            
      DOMPMTPEChargeDatum *chdatum
         = dynamic_cast<DOMPMTPEChargeDatum*>(fSinglePEChargeArray->First()); 

      Double_t charge = chdatum->GetCharge();
      Double_t norm = GetNormalization(charge);
      Double_t par[4];
      par[0] = norm;
      par[1] = fgkSigma; 
      par[2] = fgkT0; 
      par[3] = fgkGround; 

      TString fn("WaveForm_evt");
      fn += GetCurrentEventNumber();
      fWaveForm = new TF1(fn, GaussianWaveForm, 0, fgkBinMax, 4); 
      fWaveForm->SetLineWidth(2);
      fWaveForm->SetLineColor(kBlue);
      fWaveForm->SetNpx(500);
      fWaveForm->SetParameters(par);
      fWaveFormArray.Add(fWaveForm);
      fCanvas->cd();
      static Int_t flag = 0;
      if (flag == 0 ) {
         fWaveForm->Draw();
         flag = 1;
      } else {
         fWaveForm->Draw("same");
      }
      fNormHist->Fill(norm);
      
      cerr << " DOMPMTWaveForm::Process: normalization = " << norm << endl;

   } else if (fSinglePEChargeArray->GetEntries() > 1) {

      cerr << "DOMPMTWaveForm::Process: "
           << "Sorry, multi pe charge responce is not yet implimented!" << endl; 

   } else {

      cerr << "DOMPMTWaveForm::Process: "
           << "No pe!!!" << endl;

   }
   fWaveFormArray.SetOwner();
   
}

//=====================================================================
//* Finish ------------------------------------------------------------

void DOMPMTWaveForm::Finish()
{
   fCanvas1 = new TCanvas("NormHist", "Normarization", 10, 10, 400, 400);
   fCanvas1->cd();
   fNormHist->Draw();

}

